#include "prjlibs-c/standards.h"
#include <errno.h>
#include <signal.h>
#include <time.h>
#include <sys/time.h>

#include "skalibs/include/stddjb.h"
#include "prjlibs-c/constants.h"
#include "die.h"
#include "timeout.h"

static int flag_timeout, flag_timelimit;
static long timeout;
static time_t timelimit;

void timeout_setup(void) {
  unsigned long l;
  unsigned int i;
  char const* var;
  var=env_get("TIMEOUT");
  flag_timeout=(var!=null);
  if (flag_timeout) {
    i=scan_ulong(var, &l);
    if (i==0 || var[i]!='\0')
      strerr_die3x(100, PROG, ": $TIMEOUT", err_malformed);
    timeout=l;
    if (timeout<(long)0 || l!=(unsigned long)timeout)
      die_overflow();
  }
  var=env_get("TIMELIMIT");
  flag_timelimit=(var!=null);
  if (flag_timelimit) {
    time_t tmp;
    i=scan_ulong(var, &l);
    if (i==0 || var[i]!='\0')
      strerr_die3x(100, PROG, ": $TIMEOUT", err_malformed);
    timelimit=time(null);
    tmp=timelimit+(time_t)l;
    if (tmp<timelimit || (unsigned long)(tmp-timelimit)!=l)
      die_overflow();
    timelimit=tmp;
  }
  timeout_reset();
}

void timeout_reset(void) {
  time_t t;
  struct itimerval timer;
  if (!flag_timeout && !flag_timelimit) return;
  t=time(null);
  if (flag_timelimit) {
    if (timelimit<=t) raise(SIGALRM);
    timer.it_value.tv_sec=timelimit-t;
    if (flag_timeout && timer.it_value.tv_sec>timeout)
      timer.it_value.tv_sec=timeout;
  } else
    timer.it_value.tv_sec=timeout;
  timer.it_value.tv_usec=0;
  timer.it_interval.tv_sec=0;
  timer.it_interval.tv_usec=0;
  if (setitimer(ITIMER_REAL, &timer, null)!=0)
    strerr_die4sys(errstat, PROG, err_unable, err_timer, ": ");
}
